/* corr2v5d.c */

/* Vis5D version 4.2 */

/*
 *      This program reads in the correction table generated by the
*       tracker callibration program for the CAVE / Idesk , and 
*       generates a v5d file. This v5d file can then be visualized 
*       by the CAVE5D application in the CAVE / Idesk non collaboratively
*       or by CAVE6D for collaborative visualization. 
 */



#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "binio.h"
#include "v5d.h"

  struct files{
	char name[64];  
	int time;
	int date;
  };

/*
 *  This is the main conversion function.  The two filename arguments are
 *  obtained from the command line.
 *  Input:  infile - name of input file - the corresponding correction table 
 *  outfile - name of output file in v5d format
 */
 
static int convert( struct files *infiles, char *outfile,int numOfInputFiles )
{
   float *g;
   FILE *f;
   int it, iv, ir, ic, il,val;
   char *str;
   float u1,u2,v1,v2,w1,w2;
   int k;
   int j;
   

   int NumTimes;                       /* number of time steps */
   int NumVars;                       /* number of variables */
   int Nr , Nc , Nl[MAXVARS];           /* size of 3-D grids */
   char VarName[MAXVARS][10];         /* names of variables */
   int TimeStamp[MAXTIMES];           /* real times for each time step */
   int DateStamp[MAXTIMES];           /* real dates for each time step */
   int CompressMode;                  /* number of bytes per grid */
   int Projection;                    /* a projection number */
   float ProjArgs[100];               /* the projection parameters */
   int Vertical;                      /* a vertical coord system number */
   float VertArgs[MAXLEVELS];         /* the vertical coord sys parameters */
   float rowMin, rowMax, colMin, colMax, levelMin, levelMax;
   float rowcolIncr;
   float levelIncr;


   /* taking the initial values from the first input file */
   f = fopen( infiles[0].name, "r" );
   if (!f) {
      printf("Error: couldn't open %s for reading\n", infiles[0].name );
      exit(1);
   }

  
   str = (char *)malloc(256 * sizeof(char));


  fgets(str,32,f);
  sscanf(str,"%f ",&rowcolIncr);
    fgets(str,32,f);
  sscanf(str,"%f  %f",&colMin,&colMax); 
    fgets(str,32,f);
  sscanf(str,"%f %f",&levelMin,&levelMax);
  fgets(str,32,f);
  sscanf(str,"%f %f",&rowMin,&rowMax);
  

  levelIncr = rowcolIncr;

  printf( " The grid is from \n row : %f   %f \n col: %f  %f \n level: %f %f \n",rowMin, rowMax, colMin, colMax, levelMin, levelMax); 
   /* use fscanf(), fgets(), etc. to read your header if in ASCII, else use */
   /* fread() to read binary data. */
   NumTimes=numOfInputFiles;
   NumVars=4;
   Nr = (rowMax - rowMin)/rowcolIncr+1;  /* Number of rows*/
   Nc = (colMax - colMin)/rowcolIncr + 1;  /* Number of coloums*/
   Nl[0]=Nl[1] = Nl[2] = Nl[3] = (levelMax - levelMin)/levelIncr +1;  /* Number of levels for each variable */
  

   printf( " \n No of rows: %d \n No. of coloumns: %d \n No. of levels: %d \n",Nr, Nc, Nl[0]);

   strcpy(VarName[0],"U");
   strcpy(VarName[1],"V");
   strcpy(VarName[2],"W");
   strcpy(VarName[3],"T");
  
   CompressMode = 4;
   Projection = 0;
   ProjArgs[0] = rowMax;
   ProjArgs[1] = colMax;
   ProjArgs[2] = rowcolIncr;
   ProjArgs[3]= rowcolIncr;
   Vertical = 0;  /* kind of vertical coordinate .. 0- generic equally spaced*/
   VertArgs[0]=levelMin;  /*height of bottom level*/
   VertArgs[1]=levelIncr;  /* spacing between vertical coordinate..*/



 fclose(f);

for(j = 0;j< numOfInputFiles;j++)
{
   TimeStamp[j]=infiles[j].time;
   DateStamp[j]=infiles[j].date;
}

   /* use the v5dCreate call to create the v5d file and write the header */
   if (!v5dCreate( outfile, NumTimes, NumVars, Nr, Nc, Nl,
                   VarName, TimeStamp, DateStamp, CompressMode,
                   Projection, ProjArgs, Vertical, VertArgs )) {
      printf("Error: couldn't create %s\n", outfile );
      exit(1);
   }

   /*** YOU MAY CALL v5dSetLowLev() OR v5dSetUnits() HERE. SEE README FILE ***/

   /* allocate space for grid data */
   {
      int maxnl, i;
      maxnl = Nl[0];
      for (i=1;i<NumVars;i++) {
         if (Nl[i]>maxnl)
           maxnl = Nl[i];
      }
      g = (float *) malloc( Nr * Nc * maxnl * sizeof(float) );
      if (!g) {
         printf("Error: out of memory\n");
         exit(1);
      }
   }


   for (it=0;it<NumTimes;it++) {

      for (iv=0;iv<NumVars;iv++) {

#define G(ROW, COLUMN, LEVEL)   g[ (ROW) + ((COLUMN) + (LEVEL) * Nc) * Nr ]

  /* example using stdio: */
   f = fopen( infiles[it].name, "r" );
   if (!f) {
      printf("Error: couldn't open %s for reading\n", infiles[it].name );
      exit(1);
   }

 for(k=0;k<4;k++)
      fgets(str,32,f);
 
  for (ic=Nc-1; ic>=0; ic--) {
    for (il=0; il<Nl[iv]; il++) {
        for (ir=Nr-1; ir>=0; ir--) {
	      fgets(str,64,f);
	      sscanf(str,"%f %f %f %f %f %f ",&u1,&w1,&v1,&u2,&w2,&v2);
	      if( iv==0) 
		      G(ir,ic,il) = (u2-u1)*50;
	           else if(iv==1) 
			   G(ir,ic,il) = (v2-v1)*50;
	              else if(iv==2) 
			      G(ir,ic,il) = (w2-w1)*50;
	                 else if(iv==3) {
				          G(ir,ic,il) = (float )sqrt((double )((u2-u1)*(u2-u1)+(v2-v1)*(v2-v1)+(w2-w1)*(w2-w1)));
					  /*  printf(" %f \n",  G(ir,ic,il)); */ 
			 }
	                else printf( "I did not get the no. of variables properly \n");

      }
    }
  }

          fclose(f);
       
         /* Write data to v5d file. */
         if (!v5dWrite( it+1, iv+1, g )) {
            printf("Error while writing grid.  Disk full?\n");
            exit(1);
         }
      }

   }

   v5dClose();
  
}



/*
 * Main.  You shouldn't have to touch this.
 */
int main( int argc, char *argv[] )
{

  int numOfInputFiles;
  char tempName[64];
 int tempDate;
 int tempTime;
 int i;


struct files *inputFiles;
 
/*
   if (argc<3) {
      printf("Usage:\n");
      printf("   %s infile outfile \n", argv[0]);
   }
   else {
      printf("Input file: %s\n", argv[1] );
      printf("Output file: %s\n", argv[2] );
      fflush(stdout);
      convert( argv[1], argv[2]);
   }

  */

   printf("\n CORR2V5D   -  You would need to specify input files .. \n Enter files  for each instance of data collection\n\n Enter the number of data files: ");
   
   scanf("%d",&numOfInputFiles);
   inputFiles = (struct files *)(malloc(sizeof(struct files)*numOfInputFiles)); 
   for(i=0;i< numOfInputFiles;i++)
	   {
		   printf("\n Enter name of Input File %d: ",i+1);
		   scanf("%s",&tempName);
		   printf(" Enter the date of the data collected (YYDDD) ");
		   scanf("%d",&tempDate);
		   printf(" Enter the time (HHMMSS) ");
		   scanf("%d",&tempTime);

		   strcpy(inputFiles[i].name,tempName);
		   inputFiles[i].date = tempDate;
		   inputFiles[i].time = tempTime;
	   }
   
   printf("\n Enter the name of the Output file to be generated :");
   scanf("%s",&tempName);
   convert(inputFiles,tempName,numOfInputFiles);

   return 0;

}

